; DOMINUS DEUS SABAOTH (2017) for realtime Csound5/6 - by Arthur B. Hunkins
;  requires a MIDI device with 16 knobs/sliders and 8 keys/buttons/pads (notes)
;   Set all sliders to 0 prior to Start.
;   Change fundamental pitch (key/button/pad), if desired, immediately after Start.
;   If audio glitches occur, reduce the value of sr (Sample Rate,
;    see below) to 44100 or 32000 until glitching disappears.
; See performance score (and setup instructions) in Dominus.txt

<CsoundSynthesizer>
<CsOptions>

; for Windows, Mac
-odac -M0 -m0d --expression-opt -b512 -B2048 -+raw_controller_mode=1
; for Linux
;-odac -+rtaudio=alsa -+rtmidi=alsa -M hw:1,0 -m0d --expression-opt -b128 -B2048 -+raw_controller_mode=1

</CsOptions>
<CsInstruments>

sr      = 48000 ; Sample Rate  
ksmps   = 10
nchnls  = 2

#define CHAN	#1# ; 0 = CC7, CHAN 1-16 
#define CTRL1	#20# ; bank1 - first # (of 8) controllers; valid = 0 - 120
#define CTRL2	#28# ; bank2 - first # (of 8) controllers; valid = 0 - 120
#define NOTE    #60# ; first # of 8 MIDI notes (keys/buttons)
#define NOTECHN #1# ; CHAN for notes when CHAN = 0

        massign 0, 0
gitemp  ftgen 2, 0, 513, -16, 50, 513, 5, sr * .5

        instr 1

kcps    init    0
ichan   =       $CHAN
ichan   =       (ichan == 0? $NOTECHN: ichan)
kstat,kchan,knote,kd2 midiin
        if ((kstat == 0) || (kchan != ichan)) goto skip               
        if ((kstat != 144) || (kd2 == 0)) goto skip               
        if (knote < $NOTE) || (knote > $NOTE + 7) goto skip
kcps    =      (knote == $NOTE? 0: kcps)
kcps    =      (knote == $NOTE + 1? .15: kcps)
kcps    =      (knote == $NOTE + 2? .3: kcps)
kcps    =      (knote == $NOTE + 3? .44: kcps)
kcps    =      (knote == $NOTE + 4? .58: kcps)
kcps    =      (knote == $NOTE + 5? .72: kcps)
kcps    =      (knote == $NOTE + 6? .86: kcps)
kcps    =      (knote == $NOTE + 7? 1: kcps)
skip:
gkcps   tab    kcps, 4, 1
gkcps   =      gkcps * .02
gkdel   ctrl7  ($CHAN = 0? 11: $CHAN), ($CHAN = 0? 7: $CTRL2 + 2), 0, 1.7
kamp    ctrl7  ($CHAN = 0? 1: $CHAN), ($CHAN = 0? 7: $CTRL1), 0, 6300
kamp    port   kamp, .01
kmod    ctrl7  ($CHAN = 0? 1: $CHAN), ($CHAN = 0? 7: $CTRL1 + 1), 0, 1
kamp2   lfo    kmod * kamp * .15, .25
a1      lfo    kamp + kamp2, 50 * gkcps 
ga1     =      a1
ga2     =      a1
      
        endin

        instr 2

kamp    ctrl7  ($CHAN = 0? 3: $CHAN), ($CHAN = 0? 7: $CTRL1 + 2), 0, 800
kamp    port   kamp, .01
kamp2   lfo    kamp * .1, .2
kmod    ctrl7  ($CHAN = 0? 4: $CHAN), ($CHAN = 0? 7: $CTRL1 + 3), 0, 200
kmod    port   kmod, .05
kcar    lfo    kmod, 50.18 * gkcps
gkrnd   ctrl7  ($CHAN = 0? 12: $CHAN), ($CHAN = 0? 7: $CTRL2 + 3), 0, 1.2 
krand   rspline 0, gkrnd * .5, 1, 1.5
a1      lfo    kamp + kamp2, (50.23 * gkcps) + kcar + krand 
a2      delay  a1 * gkdel, .2, 1
a3      delay  a1 * gkdel, .25, 1
gkpan   ctrl7  ($CHAN = 0? 14: $CHAN), ($CHAN = 0? 7: $CTRL2 + 5), 0, .5, 1
a5,a6,a7,a8 pan a2 * 1.8, .5 + gkpan, 1, 1, 1
a9,a10,a7,a8 pan a3 * 2.7, .5 - gkpan, 1, 1, 1
gkpan2  ctrl7  ($CHAN = 0? 15: $CHAN), ($CHAN = 0? 7: $CTRL2 + 6), 0, 2.8, 1
ga1     =      ga1 + a1 + a5 + a9 + (a2 * gkpan2 * .8)
ga2     =      ga2 + a1 + a6 + a10 + (a3 * gkpan2)
      
        endin

        instr 3

kamp    ctrl7  ($CHAN = 0? 5: $CHAN), ($CHAN = 0? 7: $CTRL1 + 4), 0, 1575
kmod    ctrl7  ($CHAN = 0? 6: $CHAN), ($CHAN = 0? 7: $CTRL1 + 5), 0, 150
kmod    port   kmod, .05
kcar    lfo    kmod, 101 * gkcps
krand   rspline 0, gkrnd * 2.6, .5, .9 
a1      lfo    kamp, (201 * gkcps) + kcar + krand 
a2      delay  a1 * gkdel, .3, 1
a3      delay  a1 * gkdel, .35, 1
a5,a6,a7,a8 pan a2 * 1.8, .5 - gkpan, 1, 1, 1
a9,a10,a7,a8 pan a3 * 2.7, .5 + gkpan, 1, 1, 1
ga1     =      ga1 + a1 + a5 + a9 + (a2 * gkpan2)
ga2     =      ga2 + a1 + a6 + a10 + (a3 * gkpan2 * .8)
      
        endin

        instr 4

kamp    ctrl7  ($CHAN = 0? 7: $CHAN), ($CHAN = 0? 7: $CTRL1 + 6), 0, 495
kmod    ctrl7  ($CHAN = 0? 8: $CHAN), ($CHAN = 0? 7: $CTRL1 + 7), 0, 800
kmod    port   kmod, .05
kcar    lfo    kmod, 302 * gkcps
krand   rspline 0, gkrnd * 2.3, .7, 1.1 
a1      lfo    kamp, (402.5 * gkcps) + kcar + krand
a2      delay  a1 * gkdel, .4, 1
a3      delay  a1 * gkdel, .45,1 
a5,a6,a7,a8 pan a2 * 1.8, .5 + gkpan, 1, 1, 1
a9,a10,a7,a8 pan a3 * 2.7, .5 - gkpan, 1, 1, 1
ga1     =      ga1 + a1 + a5 + a9 + (a2 * gkpan2 * .8)
ga2     =      ga2 + a1 + a6 + a10 + (a3 * gkpan2)
      
        endin

        instr 5

kamp    ctrl7  ($CHAN = 0? 9: $CHAN), ($CHAN = 0? 7: $CTRL2), 0, 250
kmod    ctrl7  ($CHAN = 0? 10: $CHAN), ($CHAN = 0? 7: $CTRL2 + 1), 0, 1800
kmod    port   kmod, .05
kcar    lfo    kmod, 401.73 * gkcps
krand   rspline 0, gkrnd * 1.2, .8, 1
kwave   ctrl7  ($CHAN = 0? 13: $CHAN), ($CHAN = 0? 7: $CTRL2 + 4), .5, .78
kwave   port   kwave, .01
a1      vco2   kamp, (602.24 * gkcps) + kcar + krand, 4, kwave 
a2      delay  a1 * gkdel, .5, 1
a3      delay  a1 * gkdel, .55, 1
a5,a6,a7,a8 pan a2 * 1.8, .5 - gkpan, 1, 1, 1
a9,a10,a7,a8 pan a3 * 2.7, .5 + gkpan, 1, 1, 1
ga1     =      ga1 + a1 + a5 + a9 + (a2 * gkpan2)
ga2     =      ga2 + a1 + a6 + a10 + (a3 * gkpan2 * .8)
      
        endin

        instr 6
      
kamp    ctrl7  ($CHAN = 0? 16: $CHAN), ($CHAN = 0? 7: $CTRL2 + 7), 0, 1
kamp2   tab    kamp, 2, 1
kamp2   port   kamp2, .1
a1      atonex ga1, kamp2, 12   
a2      atonex ga2, kamp2, 12
kamp3   tab    kamp, 3, 1
kamp3   port   kamp3, .5
        outs   a1 * kamp3, a2 * kamp3
      
        endin
       
</CsInstruments>
<CsScore>

f 1 0 513 16 0 513 1.5 1
f 3 0 513 -5 1 500 4 11 1
f 4 0 513 -16 50 513 1 75
i 1 0 3600
i 2 0 3600
i 3 0 3600
i 4 0 3600
i 5 0 3600
i 6 0 3600

</CsScore>
</CsoundSynthesizer>